//************************************************
// Nixie Tube Clock - MSP430 version
//  ** last updated 04-28-13
//
// Copyright (c) 2012 by Derek Tombrello
//
// v1.2 - changed to external temperature sensor
//        instead of MSP430 internal sensor
//
//************************************************

#include <msp430.h>							// designed for 2452
#include "thermistor.h"

#ifndef TIMER0_A1_VECTOR
#define TIMER0_A1_VECTOR    TIMERA1_VECTOR
#define TIMER0_A0_VECTOR    TIMERA0_VECTOR
#endif

#define TEMP_SENSE 		BIT5
#define HR_SWITCH 		BIT6
#define MIN_SWITCH 		BIT7

#define HR10_ENABLE 	BIT0
#define HR01_ENABLE 	BIT1
#define COLON 			BIT4
#define MIN10_ENABLE 	BIT2
#define MIN01_ENABLE 	BIT3

#define DIGIT_PINS 		(BIT0 | BIT1 | BIT4 | BIT2 | BIT3)

enum {TIME, TEMP} mode = TIME;
unsigned int temp = 32;
unsigned int hh = 12, mm = 0, ss = 0;
unsigned char hr_press = 0, min_press = 0;
unsigned int time_count = 0, mode_count = 0;

void ConfigClocks(void);
void ConfigPins(void);
void display(void);
inline void update_switches(void);
void FaultRoutine(void);

void main(void)
{
	ConfigClocks();
	ConfigPins();
	initThermistor();

	while (1)
	{

		if (min_press)						// advance minutes
		{
			mode = TIME;					// always display time while setting
			mode_count = 0;					// and reset counter
			ss = 0;
			mm++;
			if (mm > 59) mm = 0;
			min_press = 0;
		}

		if (hr_press)						// advance hours
		{
			mode = TIME;					// always display time while setting
			mode_count = 0;					// and reset counter
			hh++;
			if (hh > 12) hh = 1;
			hr_press = 0;
		}


		if (mode_count > 9)							// every 10 seconds
		{
			mode = (mode == TIME) ? TEMP : TIME;	// swap between time and temp display
			if (mode == TEMP) temp = readTemp();	// take a temperature reading
			mode_count = 0;
		}

		display();
	}
}

void ConfigClocks(void)
{
	WDTCTL = WDTPW + WDTHOLD;          		// Stop watchdog timer

	if (CALBC1_1MHZ ==0xFF || CALDCO_1MHZ == 0xFF)
		FaultRoutine();	         			// If calibration data is erased
 				               	   	   		// run FaultRoutine()
	BCSCTL1 = CALBC1_1MHZ; 		   			// Set range
	DCOCTL = CALDCO_1MHZ;  		   			// Set DCO step + modulation
	BCSCTL3 |= LFXT1S_0 + XCAP_3;           // LFXT1 = 32768 Hz & 12.5pf cap
	while (IFG1 & OFIFG)
	{
		IFG1 &= ~OFIFG;                     // Clear OSCFault flag
		__delay_cycles(100000);
	}										// loop until it stays cleared
	BCSCTL2 = 0;  							// MCLK & SMCLK = DCO/1

	// Setup watchdog timer as an interval timer with 512 interrupts per second
	WDTCTL = WDTPW | WDTTMSEL | WDTCNTCL | WDTSSEL  | WDTIS1 | WDTIS0;
	IE1 |= WDTIE;   						// Enable the watchdog timer interrupt

//	bis_SR_register(GIE); 					// Enter interrupts
	__enable_interrupt();

}

void ConfigPins(void)
{
	P1SEL = 0;
	P1DIR = DIGIT_PINS;						// digit pins = output / switch pins = input
	P1OUT = 0;
	P1REN = HR_SWITCH | MIN_SWITCH;
	P1OUT = HR_SWITCH | MIN_SWITCH;			// enable pull-up resistors on switch pins
	P2DIR = 0xFF; 							// P2.x = output
	P2OUT = 0;
}

void display()
{
	int temp_copy = 0, digit = 0;

	if (mode == TIME)
	{
		digit = hh / 10;					// display hours 10's digit
		if (digit > 0)						// skip if no hours 10's digit
		{
			P2OUT = digit;
			P1OUT |= HR10_ENABLE;
			__delay_cycles(100);
			P1OUT &= ~HR10_ENABLE;
		}

		digit = hh % 10;					// display hours 1's digit
		P2OUT = digit;
		P1OUT |= HR01_ENABLE;
		__delay_cycles(100);
		P1OUT &= ~HR01_ENABLE;

		digit = mm / 10;					// display minutes 10's digit
		P2OUT = digit;
		P1OUT |= MIN10_ENABLE;
		__delay_cycles(100);
		P1OUT &= ~MIN10_ENABLE;

		digit = mm % 10;					// display minutes 1's digit
		P2OUT = digit;
		P1OUT |= MIN01_ENABLE;
		__delay_cycles(100);
		P1OUT &= ~MIN01_ENABLE;
	}

	if (mode == TEMP)
	{
		P1OUT &= ~COLON;					// extinguish colon for temp display
		temp_copy = temp;

		if (temp > 99)						// if temperature is over 100 degrees
		{
			digit = 1;						// display leading 100's digit
			P2OUT = digit;
			P1OUT |= HR01_ENABLE;
			__delay_cycles(100);
			P1OUT &= ~HR01_ENABLE;
			temp_copy = temp - 100;
		}

		digit = temp_copy / 10;				// display temperatures 10's digit
		if (digit > 0)						// unless under 10 degrees - not bloody likely!
		{
			P2OUT = digit;
			P1OUT |= MIN10_ENABLE;
			__delay_cycles(100);
			P1OUT &= ~MIN10_ENABLE;
		}

		digit = temp_copy % 10;				// display temperatures 1's digit
		P2OUT = digit;
		P1OUT |= MIN01_ENABLE;
		__delay_cycles(100);
		P1OUT &= ~MIN01_ENABLE;
	}
}

inline void update_switches(void)
{
   static unsigned char state[2] = {0xFF, 0xFF}; // Current debounce status

   state[0] = (state[0] << 1) | ((P1IN & HR_SWITCH) ? 1 : 0);
   state[1] = (state[1] << 1) | ((P1IN & MIN_SWITCH) ? 1 : 0);

   if (state[0] == 0x80) hr_press = 1;
   if (state[1] == 0x80) min_press = 1;

}

#pragma vector=WDT_VECTOR
__interrupt void wdt_interrupt(void)
{
	time_count++;

	if (time_count > 511)
	{										// increment clock time
		ss++;
		if (ss > 59)
		{
			ss = 0;
			mm++;
			if (mm > 59)
			{
				mm = 0;
				hh++;
				if (hh > 12) hh = 1;
			}
		}
		time_count = 0;
		mode_count++;
	}

											// every half second, toggle colon
	if ((mode == TIME) && (time_count == 0 || time_count == 256))
		P1OUT ^= COLON;

	update_switches();						// test for time-set switch press
}

void FaultRoutine(void)
 {
   for(;;)			            			// TRAP
   {										// display 77 on error
		P2OUT = 7;
		P1OUT |= HR10_ENABLE;
		__delay_cycles(50);
		P1OUT &= ~HR10_ENABLE;

		P2OUT = 7;
		P1OUT |= HR01_ENABLE;
		__delay_cycles(50);
		P1OUT &= ~HR01_ENABLE;
   }
 }
